# WIZARD_GROUP PBS
# WIZARD_NAME  node_sub
# WIZARD_DESCRIPTION template for a node sub
# WIZARD_ON

print <<'EOT' ;
sub NodeSub
{
my (
  $dependent_to_check
, $config
, $tree
, $inserted_nodes
) = @_ ;

}

EOT

1;

