=head1 PBSFILE USER HELP

Test configuration correctness

=head2 Top rules

=over 2 

=item * '*.o'

=back

=cut

AddConfig 'MULTIPLE_CC' => 'miner_2_multiple_cc' ;

#~ AddConfig 'MULTIPLE_CC::FORCE::LOCKED' => 'miner_2_multiple_cc' ;
