
AddRule 'test1', ['all' => 'HERE', 'A'] ;
AddRule 'test2', ['HERE' => 'x.z'] ;
AddRule 'test3', ['A' => 'x.z'] ;
AddRule 'test4', ['this.lib' => 'HERE', 'lib.z'] ;
AddRule 'test5', ['all' => 'lib.h', 'HERE'] ;
AddRule 'test6', ['x.z' => 'all'] ;
AddRule 'test7', ['lib.h' => 'HERE'] ;
